@@ -22,6 +22,7 @@ import           Hakyll.Core.Item
 data FileType
     = Binary
     | Css
+    | DocBook
     | Html
     | LaTeX
     | LiterateHaskell FileType
@@ -39,6 +40,7 @@ fileType :: FilePath -> FileType
 fileType = uncurry fileType' . splitExtension
   where
     fileType' _ ".css"      = Css
+    fileType' _ ".dbk"      = DocBook
     fileType' _ ".htm"      = Html
     fileType' _ ".html"     = Html
     fileType' f ".lhs"      = LiterateHaskell $ case fileType f of
